

							
/******************************************************************************************
 *                                                                                        *
 * Examples2 : GLCD_Touch Screen(ET-TFT240320TP-3.2 Rev.C) For "ET-ARM STAMP LPC2138"     *
 *																						  *
 *                            DISPLAY  LANDSCAPE(Hor.)                                    *
 *																						  *
 *	    MODE : Interface GLCD Data SPI MODE (USE Device ID=0(IM0=0))                      * 
 *																						  *
 ******************************************************************************************
 *                                                                                        *
 * Target MCU     : Philips ARM7-LPC2138                                                  *
 *       	      : X-TAL : 19.6608 MHz                                                   *
 *                : PLL Setup = M(3),P(2)	                               	 	          *
 *                : Run Speed 58.9824 MHz (With PLL)(19.6608MHz x 3 = 58.9824 MHz)        *
 *                : VPB Clock(pclk) = CPU Clock = 58.9824MHz                              *
 *                : CPU Clock(cclk) = 58.9824 MHz                                         *
 *                                                                                        *
 * Keil Editor    : uVision3 V3.62C                                                       *
 * Compiler       : RealView MDK-ARM 3.22a                                                *
 * Create By      : Mr. Sittiphol Yooyod (WWW.ETT.CO.TH)                                  *
 * Last Update    : 26/April/2011                                                        *
 *																				          *
 * Port Interface :  -- MCU-LPC2138--	  		       --LCD-ET-TFT240320TP-3.2 Rev.C--   *
 *                                                                                        *
 *     Contrl GLCD:    P1.18                 Connect          CS-H			     	      *
 *                     P1.22                 Connect          RES-H(Reset Active=0)       *
 *                     P1.23                 Connect          BL-H(LCD-Back Light(ON=1)   *
 *                                                                                        *
 *     SPI GLCD   :    P0.4/SCK              Connect          WR/SCL-H                    *
 *					   P0.6/MOSI             Connect          SDI-H                       *     																           *
 *   																				      *
 *   																				      *
 *  Touch Sceen	  :    PORT 1.26(MISO-in)	 Connect 		  MISO					      *
 *                     PORT 1.27(MOSI-out)	 Connect 	      MOSI					      *
 *                     PORT 1.28(SCLK-out)   Connect 		  SCLK					      *
 *                     PORT 1.29(BUSY-in)	 Connect 		  BUSY(No USE)			      *
 *                     PORT 1.30(PEN-in)     Connect 	      PEN					      *
 *                     PORT 1.31(TC_CS-out)	 Connect   	      TC_CS			    	      *
 *                                                                                        *
 ******************************************************************************************/




#include "LPC213x.h" 	                             
#include "stdio.h"


/******************************************************
 *			 Font 7x11(English)				          *
 *											          *
 * -ASCII fonts from 0x20 ~ 0x7F(DEC 32 ~ 126)     	  *
 *  composed of following characters 			      *
 *                                                    * 
 * -XSize : 7  pixcels							      *
 * -YSize : 11 pixcels						       	  *
 *											       	  *
 *   ....... <- b0, Blank, Not used				      *
 *   ....... <- b1, Blank, Not used				      *
 *   ....... <- b2, Blank, Not used				      *
 *   ....... <- b3, Blank, Not used				      *
 *   ....... <- b4, Blank, Not used				      *
 *   ..End.. <- b5(2nd byte)					      *
 *   ..OO... <- b6(2nd byte)					      *
 *   ..OO... <- b7(2nd byte)					      *
 *   ..OO... <- b0(1st byte)					      *
 *   ..OO... <- b1(1st byte)					      *
 *   ..OO... <- b2(1st byte)					      *
 *   ..OO... <- b3(1st byte)					      *
 *   ..OO... <- b4(1st byte)				 	      *
 *   ..OO... <- b5(1st byte)					      *
 *   ..OO... <- b6(1st byte)					      *
 *   .Start..<- b7(1st byte)					      *
 *												      *
 ******************************************************/

//7x11 ascii font, each character takes 14 bytes, total 1330 bytes, draw from left bottom to left top

const char ascii_7x11[95][14] = {                                                       
  {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x20 space
  {0x00, 0x00, 0x07, 0xc0, 0xdf, 0xe0, 0xdf, 0xe0, 0x07, 0xc0, 0x00, 0x00, 0x00, 0x1f}, //0x21 !
  {0x00, 0x00, 0x00, 0xe0, 0x00, 0xe0, 0x00, 0x00, 0x00, 0xe0, 0x00, 0xe0, 0x00, 0x1f}, //0x22 "
  {0x00, 0x00, 0x1b, 0x00, 0x7f, 0xc0, 0x1b, 0x00, 0x7f, 0xc0, 0x1b, 0x00, 0x00, 0x1f}, //0x23 #
  {0x13, 0x00, 0x27, 0x80, 0x24, 0x80, 0x7f, 0xc0, 0x24, 0x80, 0x38, 0x80, 0x19, 0x1f}, //0x24 $
  {0x30, 0x80, 0x19, 0x40, 0x2d, 0x40, 0x56, 0x80, 0x53, 0x00, 0x21, 0x80, 0x00, 0x1f}, //0x25 %
  {0x38, 0x00, 0x7d, 0x80, 0x47, 0xc0, 0x5e, 0x40, 0x7b, 0xc0, 0x21, 0x80, 0x58, 0x1f}, //0x26 &
  {0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x00, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x27 '
  {0x00, 0x00, 0x00, 0x00, 0x3f, 0x80, 0x7f, 0xc0, 0xc0, 0x60, 0x80, 0x20, 0x00, 0x1f}, //0x28 (
  {0x00, 0x00, 0x80, 0x20, 0xc0, 0x60, 0x7f, 0xc0, 0x3f, 0x80, 0x00, 0x00, 0x00, 0x1f}, //0x29 )
  {0x00, 0x00, 0x24, 0x80, 0x3f, 0x80, 0x0e, 0x00, 0x3f, 0x80, 0x24, 0x80, 0x00, 0x1f}, //0x2a *
  {0x04, 0x00, 0x04, 0x00, 0x1f, 0x00, 0x1f, 0x00, 0x04, 0x00, 0x04, 0x00, 0x00, 0x1f}, //0x2b +
  {0x00, 0x00, 0x80, 0x00, 0xe0, 0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x2c ,
  {0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0x00, 0x1f}, //0x2d -
  {0x00, 0x00, 0x00, 0x00, 0xc0, 0x00, 0xc0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x2e .
  {0xc0, 0x00, 0xf0, 0x00, 0x3c, 0x00, 0x0f, 0x00, 0x03, 0xc0, 0x00, 0xe0, 0x00, 0x3f}, //0x2f /
  {0x7f, 0xc0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0xff, 0xe0, 0x7f, 0xdf}, //0x30 0
  {0x00, 0x00, 0x80, 0x80, 0x80, 0xc0, 0xff, 0xe0, 0xff, 0xe0, 0x80, 0x00, 0x00, 0x1f}, //0x31 1
  {0xe0, 0xc0, 0xf0, 0xe0, 0x98, 0x20, 0x8c, 0x20, 0x84, 0x20, 0x87, 0xe0, 0x83, 0xdf}, //0x32 2
  {0x60, 0xc0, 0xe0, 0xe0, 0x84, 0x20, 0x84, 0x20, 0x84, 0x20, 0xff, 0xe0, 0x7b, 0xdf}, //0x33 3
  {0x38, 0x00, 0x3e, 0x00, 0x27, 0x80, 0x21, 0xe0, 0xff, 0xe0, 0xff, 0xe0, 0x20, 0x1f}, //0x34 4
  {0x83, 0xe0, 0x83, 0xe0, 0x82, 0x20, 0x82, 0x20, 0xc2, 0x20, 0x7e, 0x20, 0x3c, 0x3f}, //0x35 5
  {0x7f, 0x00, 0xff, 0xc0, 0x82, 0x60, 0x82, 0x20, 0x82, 0x20, 0xfe, 0x20, 0x7c, 0x3f}, //0x36 6
  {0x00, 0x20, 0x00, 0x20, 0xe0, 0x20, 0xfc, 0x20, 0x1f, 0x20, 0x03, 0xe0, 0x00, 0xff}, //0x37 7
  {0x7b, 0xc0, 0xff, 0xe0, 0x86, 0x20, 0x86, 0x20, 0x8c, 0x20, 0xff, 0xe0, 0x7b, 0xdf}, //0x38 8
  {0x03, 0xc0, 0x87, 0xe0, 0x84, 0x20, 0xc4, 0x20, 0x64, 0x20, 0x3f, 0xe0, 0x1f, 0xdf}, //0x39 9
  {0x00, 0x00, 0x00, 0x00, 0x31, 0x80, 0x31, 0x80, 0x31, 0x80, 0x00, 0x00, 0x00, 0x1f}, //0x3a :
  {0x00, 0x00, 0x00, 0x00, 0xb1, 0x80, 0xf1, 0x80, 0x71, 0x80, 0x00, 0x00, 0x00, 0x1f}, //0x3b ;
  {0x04, 0x00, 0x0e, 0x00, 0x1b, 0x00, 0x31, 0x80, 0x60, 0xc0, 0xc0, 0x60, 0x80, 0x3f}, //0x3c <
  {0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x00, 0x0a, 0x1f}, //0x3d =
  {0x80, 0x20, 0xc0, 0x60, 0x60, 0xc0, 0x31, 0x80, 0x1b, 0x00, 0x0e, 0x00, 0x04, 0x1f}, //0x3e >
  {0x01, 0xc0, 0x01, 0xe0, 0x00, 0x20, 0xd8, 0x20, 0xdc, 0x20, 0x07, 0xe0, 0x03, 0xdf}, //0x3f ?
  {0x7f, 0x40, 0x83, 0x20, 0x83, 0x20, 0xff, 0x20, 0x80, 0x60, 0xff, 0xe0, 0x7f, 0xdf}, //0x40 @
  {0xff, 0x00, 0xff, 0x80, 0x08, 0xc0, 0x08, 0x60, 0x08, 0xc0, 0xff, 0x80, 0xff, 0x1f}, //0x41 A
  {0xff, 0xe0, 0xff, 0xe0, 0x84, 0x20, 0x84, 0x20, 0x84, 0x20, 0xff, 0xe0, 0x7b, 0xdf}, //0x42 B
  {0x7f, 0xc0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0xe0, 0xe0, 0x60, 0xdf}, //0x43 C
  {0xff, 0xe0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0xc0, 0x60, 0x7f, 0xc0, 0x3f, 0x9f}, //0x44 D
  {0xff, 0xe0, 0xff, 0xe0, 0x84, 0x20, 0x84, 0x20, 0x84, 0x20, 0x84, 0x20, 0x80, 0x3f}, //0x45 E
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x00, 0x3f}, //0x46 F
  {0x7f, 0xc0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x88, 0x20, 0xf8, 0xe0, 0xf8, 0xdf}, //0x47 G
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x00, 0x04, 0x00, 0x04, 0x00, 0xff, 0xe0, 0xff, 0xff}, //0x48 H
  {0x00, 0x00, 0x80, 0x20, 0xff, 0xe0, 0xff, 0xe0, 0x80, 0x20, 0x00, 0x00, 0x00, 0x1f}, //0x49 I
  {0x60, 0x00, 0xe0, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0xff, 0xe0, 0x7f, 0xff}, //0x4a J
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x00, 0x1f, 0x00, 0x7b, 0xc0, 0xe0, 0xe0, 0x80, 0x3f}, //0x4b K
  {0xff, 0xe0, 0xff, 0xe0, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x1f}, //0x4c L
  {0xff, 0xe0, 0xff, 0xe0, 0x00, 0x80, 0x07, 0x00, 0x00, 0x80, 0xff, 0xe0, 0xff, 0xff}, //0x4d M
  {0xff, 0xe0, 0xff, 0xe0, 0x03, 0x80, 0x0e, 0x00, 0x38, 0x00, 0xff, 0xe0, 0xff, 0xff}, //0x4e N
  {0x7f, 0xc0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0xff, 0xe0, 0x7f, 0xdf}, //0x4f O
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x07, 0xe0, 0x03, 0xdf}, //0x50 P
  {0x7f, 0xc0, 0xff, 0xe0, 0x90, 0x20, 0xb0, 0x20, 0xe0, 0x20, 0x7f, 0xe0, 0xff, 0xdf}, //0x51 Q
  {0xff, 0xe0, 0xff, 0xe0, 0x04, 0x20, 0x04, 0x20, 0x0c, 0x20, 0xff, 0xe0, 0xf3, 0xdf}, //0x52 R
  {0x43, 0xc0, 0xc7, 0xe0, 0x86, 0x20, 0x84, 0x20, 0x8c, 0x20, 0xfc, 0x60, 0x78, 0x5f}, //0x53 S
  {0x00, 0x20, 0x00, 0x20, 0xff, 0xe0, 0xff, 0xe0, 0x00, 0x20, 0x00, 0x20, 0x00, 0x1f}, //0x54 T
  {0x7f, 0xe0, 0xff, 0xe0, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0xff, 0xe0, 0x7f, 0xff}, //0x55 U
  {0x1f, 0xe0, 0x3f, 0xe0, 0x60, 0x00, 0xc0, 0x00, 0x60, 0x00, 0x3f, 0xe0, 0x1f, 0xff}, //0x56 V
  {0x1f, 0xe0, 0xff, 0xe0, 0xe0, 0x00, 0x1c, 0x00, 0xe0, 0x00, 0xff, 0xe0, 0x1f, 0xff}, //0x57 W
  {0xc0, 0x60, 0xf1, 0xe0, 0x3f, 0x80, 0x0e, 0x00, 0x3f, 0x80, 0xf1, 0xe0, 0xc0, 0x7f}, //0x58 X
  {0x03, 0xe0, 0x07, 0xe0, 0xfc, 0x00, 0xfc, 0x00, 0x07, 0xe0, 0x03, 0xe0, 0x00, 0x1f}, //0x59 Y
  {0xc0, 0x20, 0xf0, 0x20, 0xbc, 0x20, 0x8f, 0x20, 0x83, 0xa0, 0x80, 0xe0, 0x80, 0x7f}, //0x5a Z
  {0x00, 0x00, 0xff, 0xe0, 0xff, 0xe0, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0x00, 0x1f}, //0x5b [
  {0x00, 0x20, 0x00, 0xe0, 0x03, 0xc0, 0x0f, 0x00, 0x3c, 0x00, 0xf0, 0x00, 0xc0, 0x1f}, /*0x5c \*/
  {0x00, 0x00, 0x80, 0x20, 0x80, 0x20, 0x80, 0x20, 0xff, 0xe0, 0xff, 0xe0, 0x00, 0x1f}, //0x5d ]
  {0x00, 0x80, 0x00, 0xc0, 0x00, 0x60, 0x00, 0x60, 0x00, 0xc0, 0x00, 0x80, 0x00, 0x1f}, //0x5e ^
  {0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x1f}, //0x5f _
  {0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x60, 0x00, 0xe0, 0x00, 0x80, 0x00, 0x1f}, //0x60 `
  {0x70, 0x00, 0xf9, 0x00, 0x89, 0x00, 0x89, 0x00, 0x89, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x61 a
  {0xff, 0xe0, 0xff, 0xe0, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xff, 0x00, 0x7e, 0x1f}, //0x62 b
  {0x7e, 0x00, 0xff, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xc3, 0x00, 0x42, 0x1f}, //0x63 c
  {0x7e, 0x00, 0xff, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xff, 0xe0, 0xff, 0xff}, //0x64 d
  {0x7e, 0x00, 0xff, 0x00, 0x91, 0x00, 0x91, 0x00, 0x91, 0x00, 0x9f, 0x00, 0x5e, 0x1f}, //0x65 e
  {0x04, 0x00, 0xff, 0xc0, 0xff, 0xe0, 0x04, 0x20, 0x04, 0x20, 0x04, 0x20, 0x00, 0x1f}, //0x66 f
  {0x9e, 0x00, 0xbf, 0x00, 0xa1, 0x00, 0xa1, 0x00, 0xa1, 0x00, 0xff, 0x00, 0x7f, 0x1f}, //0x67 g
  {0xff, 0xe0, 0xff, 0xe0, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x68 h
  {0x81, 0x00, 0x81, 0x00, 0xff, 0x60, 0xff, 0x60, 0x80, 0x00, 0x80, 0x00, 0x00, 0x1f}, //0x69 i
  {0x80, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xff, 0x60, 0x7f, 0x60, 0x00, 0x1f}, //0x6a j
  {0xff, 0xe0, 0xff, 0xe0, 0x18, 0x00, 0x3c, 0x00, 0x66, 0x00, 0xc3, 0x00, 0xc1, 0x1f}, //0x6b k
  {0x80, 0x20, 0x80, 0x20, 0xff, 0xe0, 0xff, 0xe0, 0x80, 0x00, 0x80, 0x00, 0x00, 0x1f}, //0x6c l
  {0xff, 0x00, 0xff, 0x00, 0x01, 0x00, 0x3f, 0x00, 0x01, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x6d m
  {0xff, 0x00, 0xff, 0x00, 0x01, 0x00, 0x01, 0x00, 0x01, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x6e n
  {0x7e, 0x00, 0xff, 0x00, 0x81, 0x00, 0x81, 0x00, 0x81, 0x00, 0xff, 0x00, 0x7e, 0x1f}, //0x6f o
  {0xff, 0x00, 0xff, 0x00, 0x21, 0x00, 0x21, 0x00, 0x21, 0x00, 0x3f, 0x00, 0x1e, 0x1f}, //0x70 p
  {0x1e, 0x00, 0x3f, 0x00, 0x21, 0x00, 0x21, 0x00, 0x21, 0x00, 0xff, 0x00, 0xfe, 0x1f}, //0x71 q
  {0x01, 0x00, 0xff, 0x00, 0xff, 0x00, 0x04, 0x00, 0x02, 0x00, 0x03, 0x00, 0x03, 0x1f}, //0x72 r
  {0x8e, 0x00, 0x9f, 0x00, 0x99, 0x00, 0x99, 0x00, 0x99, 0x00, 0xf9, 0x00, 0x71, 0x1f}, //0x73 s
  {0x01, 0x00, 0x01, 0x00, 0x7f, 0xc0, 0xff, 0xc0, 0x81, 0x00, 0x81, 0x00, 0x81, 0x1f}, //0x74 t
  {0x7f, 0x00, 0xff, 0x00, 0x80, 0x00, 0x80, 0x00, 0x80, 0x00, 0xff, 0x00, 0xff, 0x1f}, //0x75 u
  {0x1f, 0x00, 0x3f, 0x00, 0x60, 0x00, 0xc0, 0x00, 0x60, 0x00, 0x3f, 0x00, 0x1f, 0x1f}, //0x76 v
  {0x3f, 0x00, 0xff, 0x00, 0xc0, 0x00, 0x3c, 0x00, 0xc0, 0x00, 0xff, 0x00, 0x3f, 0x1f}, //0x77 w
  {0xe3, 0x00, 0xf7, 0x00, 0x1c, 0x00, 0x1c, 0x00, 0x1c, 0x00, 0xf7, 0x00, 0xe3, 0x1f}, //0x78 x
  {0x8f, 0x00, 0x9f, 0x00, 0x90, 0x00, 0x90, 0x00, 0x90, 0x00, 0xff, 0x00, 0x7f, 0x1f}, //0x79 y
  {0xc1, 0x00, 0xe1, 0x00, 0xb1, 0x00, 0x99, 0x00, 0x8d, 0x00, 0x87, 0x00, 0x83, 0x1f}, //0x7a z
  {0x00, 0x00, 0x04, 0x00, 0x7f, 0xc0, 0xe0, 0xe0, 0x80, 0x20, 0x80, 0x00, 0x00, 0x1f}, //0x7b {
  {0x00, 0x00, 0x00, 0x00, 0xff, 0xe0, 0xff, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1f}, //0x7c |
  {0x00, 0x00, 0x80, 0x20, 0xe0, 0xe0, 0x7f, 0xc0, 0x04, 0x00, 0x00, 0x00, 0x00, 0x5f}, //0x7d }
  {0x00, 0x60, 0x00, 0x20, 0x00, 0x60, 0x00, 0xc0, 0x00, 0x80, 0x00, 0xc0, 0x00, 0x5f}  //0x7e ~
};


/*########***** Define Color *****########
  #####  Color = RRRRR GGGGGG BBBBB ######
  ########************************######## */

#define no_bg               0x0001         //No Back Ground Color
#define BLACK               0x0000
#define WHITE               0xFFFF
#define RED                 0x8000
#define GREEN               0x0400
#define BLUE                0x0010
#define YELLOW           	0xFFF0
#define CYAN                0x0410
#define MAGENTA             0x8010
#define BROWN               0xFC00
#define OLIVE	            0x8400

#define BRIGHT_RED          0xF800
#define BRIGHT_GREEN        0x07E0
#define BRIGHT_BLUE         0x001F
#define BRIGHT_YELLOW       0xFFE0
#define BRIGHT_CYAN         0x07FF
#define BRIGHT_MAGENTA    	0xF81F

#define LIGHT_GRAY          0x8410
#define DARK_GRAY         	0x4208
#define LIGHT_BLUE          0x841F
#define LIGHT_GREEN         0x87F0
#define LIGHT_CYAN          0x87FF
#define LIGHT_RED         	0xFC10
#define LIGHT_MAGENTA      	0xFC1F

#define GRAY0       	    0xE71C
#define GRAY1         	    0xC618
#define GRAY2             	0xA514
#define GRAY3             	0x630C
#define GRAY4            	0x4208
#define GRAY5	            0x2104
#define	GRAY6	          	0x3186	

#define	BLUE0	            0x1086	
#define	BLUE1       		0x3188 
#define	BLUE2          	    0x4314
#define BLUE3         	    0x861C	

#define	CYAN0	            0x3D34
#define	CYAN1               0x1DF7		

#define	GREEN0              0x0200	
#define	GREEN1              0x0208	


//********** Valiable delay ***********

unsigned long timeval                 ;

//********** Valiable for Touch Screen Function GLCD ************

long dif_adc_X,dif_adc_Y              ;
long buf_adc_X[100],buf_adc_Y[100]    ;
long ave_adc_X,ave_adc_Y              ;


long tcs_ave_X[3]				      ;	// Keep Value adc Touch Point X
long tcs_ave_Y[3]                     ;	// Keep Value adc Touch Point Y
long dis_XD[3] = {32,287,160}         ; // Value refer Point X at 10% of Display(X=320:0-319) 3 Position
long dis_YD[3] = {215,120,24}         ; // Value refer Point Y at 10% of Display(Y=240:0-239) 3 Position
long divider,An,Bn,Cn,Dn,En,Fn,X,Y    ;	// Valiable for keep coefficient Calibrat and position touch Screen
//char num1,num2,num3                   ;	// Variable for keep data Dec to Ascii




/*#########****** 1.Function main ****##############*/

 void init_port(void)                    ;
 __irq void tc0 (void)                   ;
 void delay_ms(unsigned long ms)         ;


/*##########***** 2.Function Control Graphic LCD ET-TFT240320TP-3.2 Rev.C (Driver ILI9325)****#########*/

void gp_cs_hi(void)                               ;
void gp_cs_lo(void)                               ;
void gp_res_hi(void)				              ;
void gp_res_lo(void)				              ;
void gp_bl_on(void)				                  ;
void gp_bl_off(void)				              ;
unsigned char spi_rdwr(unsigned char DataByte)    ;
void gp_wr_cmm(unsigned char cmm)                 ;
void gp_wr_data(unsigned int dw)                  ;
void init_glcd(void)                              ;



/*##########***** 3.Function Control SPI Touch-Screen (ADS7846)*****#########*/

void tcs_sck_hi(void)                             ;
void tcs_sck_lo(void)                             ;
void tcs_cs_hi(void)					          ;
void tcs_cs_lo(void)					          ;
void tcs_sdo_hi(void)					          ;
void tcs_sdo_lo(void)				  	          ;
char status_touch(void)                           ;
unsigned char tcs_wr(unsigned char DataByte)      ;
void tcs_read(void)                               ;
void ave_x_y(unsigned char num)	                  ;

void touch_calibrate(void)                        ;
void set_matrix(void)                             ;
void get_point(char num)                          ;


/*##########****** 4.Function application of GLCD *******###########*/

void bg_color(long bg_color)                                                   ;
void plot_dot(unsigned long Xadd,unsigned long Yadd,unsigned long color)       ;
void plot_tab(unsigned long color)                                             ;  
void plot_mark(long x1,long y1,long x2,long y2,long color)                     ;

void text_7x11(char row,long adx,long ady,long fg_clr,long bg_clr)             ;
void lcd_printStr(char *str,long cur_x,long cur_y,long fg_color,long bg_color) ;



/* ################################################################
   ##														     ##
   ##                        Main Program                        ##
   ##														 	 ##
   ################################################################ */
 
 main(void)
  {  
    char select=0                         ;
    long chang_color = BRIGHT_YELLOW      ;


 //------------------- Start GLCD and Cribrate Touch Screen ---------------
  
   init_port()			 ;
   init_glcd()           ;
   touch_calibrate()	 ;

   plot_tab(chang_color) ;

   //----------- Start Main Program -------------	
    while(1)
	 {	  		
	  
		if(status_touch())                 	                //return (1) if tap touch screen is detected.
		 {
		 
			    get_point(20)                 ;             //Calculate Position Address Display Keep result in valiable X,Y
				
					if((X>=320)&&(X<=335)&&(Y>=15)&&(Y<=38))  //Touch Picture Home
				  {										  
                
				   bg_color(BLACK)            ;			    //Clear Screen
				   plot_tab(chang_color)      ;
				 
				 }
		 	   if((X>=320)&&(X<=330)&&(Y>=207)&&(Y<=232))    //Touch Picture Note
				 {
				   switch(select)						    //Select Color
				    {
					  case 0 : 
					           chang_color = BRIGHT_RED       ; 
							   plot_tab(chang_color)          ;
							   select = 1                     ;
						 break;


					  case 1 : 
					           chang_color = BRIGHT_GREEN     ;
							   plot_tab(chang_color)          ;
							   select   = 2                   ;

						 break;



                      case 2 : 
					           chang_color = BRIGHT_BLUE      ;
							   plot_tab(chang_color)          ;
							   select = 3                     ;

						 break;


					  case 3 : 
					           chang_color = BRIGHT_MAGENTA   ;
							   plot_tab(chang_color)          ;
							   select = 4                     ;

						 break;


				      case 4 : 
					           chang_color = BRIGHT_CYAN      ;
							   plot_tab(chang_color)          ;
							   select = 5                     ;

						 break;


					 case 5 : 
					           chang_color = BROWN           ;
							   plot_tab(chang_color)         ;
							   select = 6                    ;
						 break;


					  case 6 : 
					           chang_color = OLIVE           ;
							   plot_tab(chang_color)         ;
							   select   = 7                  ;
															 
						 break;



                      case 7 : 
					           chang_color =  LIGHT_RED       ;
							   plot_tab(chang_color)          ;
							   select = 8                     ;

						 break;


					 case 8 : 
					           chang_color =  LIGHT_GREEN     ;
							   plot_tab(chang_color)          ;
							   select = 9                     ;

						 break;


				      case 9 : 
					           chang_color =  LIGHT_BLUE      ;
							   plot_tab(chang_color)          ;
							   select = 10                    ;

						 break;

					 case 10 : 
					           chang_color =  LIGHT_GRAY      ;
							   plot_tab(chang_color)          ;
							   select = 11                    ;

						 break;

					
                      case 11 : 
					           chang_color =  RED             ;
							   plot_tab(chang_color)          ;
							   select = 12                    ;

						 break;


					 case 12 : 
					           chang_color =  GREEN           ;
							   plot_tab(chang_color)          ;
							   select = 13                    ;

						 break;


				      case 13 : 
					           chang_color =  BLUE            ;
							   plot_tab(chang_color)          ;
							   select = 14                    ;

						 break;



					  case 14 : 
					           chang_color = WHITE            ;
							   plot_tab(chang_color)          ;
							   select = 15                    ;

						 break;


					  case 15 : 
					           chang_color = BRIGHT_YELLOW    ;
							   plot_tab(chang_color)          ;
							   select = 0                     ;

						 break;

					
					}
				    while(status_touch()) ;
					delay_ms(200);
				  }



				if((X>=5)&&(X<=310)&&(Y>=5)&&(Y<=235))	   //Touch Display
				  {
			       plot_dot(X,Y,chang_color)     ; //Plot dot1
				   plot_dot(X+1,Y+1,chang_color) ; //Plot dot2
				   plot_dot(X-1,Y-1,chang_color) ; //Plot dot3
				  }

		}  
     }

}




/*################################################################
  ##                       SUB-Program                          ##
  ################################################################ */

/*#############****************************************###########
  #############*****       1.Function Main        *****###########
  #############*****************************************########## */
 
//**************************************************************  
//****** Interrupt Service routeen Timer0 For Delay(ms) ********
//************************************************************** 

  __irq void tc0 (void) 
   {
    
	 timeval++         ;
     T0IR        = 1   ;          // Clear interrupt flag
     VICVectAddr = 0   ;          // Acknowledge Interrupt 
  }


//********************************
//*********** Delay ms ***********
//********************************

void delay_ms(unsigned long ms)
 {
  timeval = 0     ;
  T0TCR   = 1     ;              // Timer0 Enable
 
  while(timeval != ms){;}
  T0TCR   = 0     ;				 // Timer0 Disable
 }


//**********************************
//*********** Initial Port *********
//**********************************

void init_port()
 {
 
//------ Set GPIO  PIN -------

  PINSEL0 = 0x00001505               ;	   //Set Port0-PIN[4..6] = SPI0 ,RxD0(P0.1) and TxD0(P0.0),Pin Other= GPIO
  PINSEL1 = 0x00000000               ;     //Set Port0-PIN[16..31]= GPIO
  PINSEL2 = 0x00000000               ;	   //Set Port1-PIN[16..31]= GPIO

 //------- Set Direction of GPIO PIN -------

  IODIR0  = 0xFFFFFFDD               ;	   //Set Port0-PIN[Other] = Output,  ;P0.1(RxD0),P0.5(MISO)=Input
  IODIR1  = 0x9BFF0000               ;	   //Set Port1-PIN[16..23]= Output	;P1.26,P1.29,P1.30=Input

//-------- Default Value ----------
  IOPIN0  = 0x00000080               ;     //Set Data Port (P0.7(SS)) = 1
  IOPIN1  = 0xC0C40000               ;	   //Set PIN Control LCD = 1 [P1.18(LCD CS),P1.22(RES),P1.23(BL),P1.30,P1.31]

 /* 	
 //--------- Initial Uart0 ----------
 
	U0LCR  = 0x83                    ;     //8 bits, no Parity, 1 Stop bit           
	U0DLL  = 192                     ;     //9600 Baud Rate @ 28.491 MHz VPB Clock        
	U0LCR  = 0x03                    ;     //DLAB = 0    
  */


 //--------------- Initial SPI0 for GLCD --------------------

    S0SPCR  = 0x00    ;
    S0SPCCR = 0x08    ;  // SCK = PCLK/SPCCR0
    S0SPCR  = 0x0038  ;	 //8 bit,CPOL=1,CPHA=1, master mode, MSB first,interrupt enabled ############# Edit ############

 //------------ Initial Timer0 -------------
  //------ Timer Counter 0 Interrupt executes each 1ms @ 60 MHz CPU Clock ------ 
 
  T0MR0        = 60000               ;    //counts = 1 ms/pclk 
  T0MCR        = 3                   ;    //Interrupt and Reset on MR0
  VICVectAddr0 = (unsigned long) tc0 ;    //set interrupt vector in 0
  VICVectCntl0 = 0x20 | 4            ;    //use it for Timer 0 Interrupt
  VICIntEnable = 0x00000010          ;    //Enable Timer0 Interrupt

}



/*##########**************************************************************************************#########
  ##########**** 2.Function Control Graphic LCD ET-TFT240320TP-3.2 Rev.C (Driver ILI9325)     ****#########
  ##########**************************************************************************************######### */

//******** Control Pin cs(P1.18=CS) ********

void gp_cs_hi(void)
 {
   IOSET1 = 0x00040000;	   //P1.18 = Hi
 }

void gp_cs_lo(void)
{
  IOCLR1 = 0x00040000 ;	   //P1.18 = Lo
}

 
//******** Control Pin res(P1.22=RES) *********
void gp_res_hi(void)
 {
   IOSET1 = 0x00400000;	   //P1.22 = Hi
 }


void gp_res_lo(void)
{
  IOCLR1 = 0x00400000 ;	   //P1.22 = Lo
}

//********* Control Back Light(P1.23=Back Light)**********

void gp_bl_on(void)
{
  IOSET1 = 0x00800000 ;	   //P1.23 = Hi(ON)
}

void gp_bl_off(void)
{
  IOCLR1 = 0x00800000 ;	   //P1.23 = Lo(OFF)
}



/****************************************************
 *******        GLCD SPI Sent Data 8 bit      *******
 ****************************************************/

unsigned char spi_rdwr(unsigned char DataByte)    
{
   S0SPDR = DataByte ;
   while(!(S0SPSR & 0x80)); //Check SPI Transfer Complete=1
   
   return S0SPDR ;
}


//***************************************************************************
//*************  Write Address Command(Index Reg.)(Use Device ID=0) *********
//***************************************************************************

void gp_wr_cmm(unsigned char cmm)
 {

    gp_cs_lo()                            ;//CS_LCD = 0

	spi_rdwr(0x70)                       ;//Sent Byte 1 = [Device ID Code:01110[0]]+[RS:0] + [R/W:0]
    spi_rdwr(0x00)                       ;//Sent Byte 2 = data 8 bit High Index Reg.: 0x00
    spi_rdwr(cmm)                        ;//Sent Byte 3 = data 8 bit Low index reg. : cmm
    
    gp_cs_hi()                           ;//CS_LCD = 1

 }
 

//******************************************************************
//************* Write data to LCD (Use Device ID=0)  ***************
//******************************************************************

void gp_wr_data(unsigned int dw)
 {	
     gp_cs_lo()                        ;     //CS_LCD = 0
  
     spi_rdwr(0x72)                    ;     //Byte 1 = [Device ID Code:01110[0]]+[RS:1] + [R/W:0]
     spi_rdwr(dw >> 8)                 ;     //Byte 2 = Data 8 bit High 
     spi_rdwr(dw)                      ;     //Byte 3 = Data 8 bit Low 
  
     gp_cs_hi()                        ;     //CS_LCD = 1
 }


 
 
//***************************************************** 
//**********     Function Initial GLCD       **********
//*****************************************************

void init_glcd(void)
 {
    unsigned long cnt ;

    //-------------- Start GLCD --------------
	 
	   
  	  gp_res_lo()           ;    // Reset GLCD 
      delay_ms(1)           ;    // Delay for 1ms.	 
      gp_res_hi()           ;    // Stop Signal Reset GLCD    
      delay_ms(1)           ;	 // Delay for 1ms while the LCD comes out of reset.
   
  
      gp_wr_cmm(0x00)       ;	 // Command Start Oscillater
      gp_wr_data(0x0001)    ;	 // Enable the oscillator.
   
      delay_ms(10)          ;    // Delay for 10ms while the oscillator stabilizes.
   

	 //----- Display Setting -----
	 	
	  gp_wr_cmm(0x01)       ;	 // Configure the output drivers.
      gp_wr_data(0x0100)    ;
      gp_wr_cmm(0x02)       ;	 // Configure the LCD A/C drive waveform.
      gp_wr_data(0x0700)    ;	 // Line inversion 0x0700
      gp_wr_cmm(0x03)       ;    // Configure Entry Mode
      gp_wr_data(0x1228)    ; 	 // Color:RGB ,Incremen Ver.& decrement Hor. address ,Address update Ver.
   

   //------ Configure and enable the LCD power supply.-------

	 //---- Power Control -----

	  gp_wr_cmm(0x07)       ;    // Enable internal operation of the LCD controller.
      gp_wr_data(0x0101)    ;	 // 0011
	  gp_wr_cmm(0x10)       ;
      gp_wr_data(0x0000)    ;  //0x00B0
      gp_wr_cmm(0x11)       ;
      gp_wr_data(0x0007)    ;   //0x0037
	  gp_wr_cmm(0x12)       ;
      gp_wr_data(0x0000)    ;  //0x011E
	  gp_wr_cmm(0x13)       ;
      gp_wr_data(0x0000)    ;  //0x1A00
	  delay_ms(20)          ;

	  gp_wr_cmm(0x10)       ;
      gp_wr_data(0x16B0)    ;
      gp_wr_cmm(0x11)       ;
      gp_wr_data(0x0037)    ;
	  delay_ms(20)          ; //50

	  gp_wr_cmm(0x12)       ;
      gp_wr_data(0x013E)    ;
	  delay_ms(20)          ;  //50

	  gp_wr_cmm(0x13)       ;
      gp_wr_data(0x1A00)    ;
      gp_wr_cmm(0x29)       ;
      gp_wr_data(0x000F)    ;      
      delay_ms(20)          ;    //50- Delay for 40ms while the power supply stabilizes.
    
   
	 
	  //------- Clear the contents of the display buffer. --------
   
      gp_wr_cmm(0x22);

      for(cnt=0;cnt<(320 * 240);cnt++)
       {
         gp_wr_data(BLACK)  ;    // Back-Ground Color Black
       }


	  gp_wr_cmm(0x20)       ;    //Command Horizontal GRAM Address Set
      gp_wr_data(0x0000)    ;	 //Address Start 0x0000 
	  gp_wr_cmm(0x21)       ;    //Command Vertical GRAM Address Set
      gp_wr_data(0x0000)    ;	 //Address Start 0x0000 

	  gp_wr_cmm(0x50)       ;  	 //Command Horizontal Address Start Position
	  gp_wr_data(0x0000)    ;  
	  gp_wr_cmm(0x51)       ;  	 //Command Horizontal Address End Position(239)
	  gp_wr_data(0x00EF)    ;  

	  
	  gp_wr_cmm(0x52)       ;  	 //Command Vertical Address Start Position
	  gp_wr_data(0x0000)    ;  
	  gp_wr_cmm(0x53)       ;  	 //Command Verticall Address End Position(239)
	  gp_wr_data(0x013F)    ;  
  

      gp_wr_cmm(0x60)       ;    // Set the number of lines to scan.
      gp_wr_data(0xA700)    ;	 // GS = 1                             ############# Edit ##############

      gp_wr_cmm(0x61)       ;    // Enable grayscale inversion of the source outputs.
      gp_wr_data(0x0001)    ;

    
   //------- GAMMA Control -------

	  gp_wr_cmm(0x30)       ; 
	  gp_wr_data(0x0007)    ;
	  gp_wr_cmm(0x31)       ; 
	  gp_wr_data(0x0403)    ;
  	  gp_wr_cmm(0x32)       ; 
	  gp_wr_data(0x0404)    ;
      gp_wr_cmm(0x35)       ; 
	  gp_wr_data(0x0002)    ;
      gp_wr_cmm(0x36)       ; 
	  gp_wr_data(0x0707)    ;
	  gp_wr_cmm(0x37)       ; 
	  gp_wr_data(0x0606)    ;
      gp_wr_cmm(0x38)       ; 
	  gp_wr_data(0x0106)    ; 
	  gp_wr_cmm(0x39)       ; 
	  gp_wr_data(0x0007)    ;
	  gp_wr_cmm(0x3C)       ; 
	  gp_wr_data(0x0700)    ;
  	  gp_wr_cmm(0x3D)       ; 
	  gp_wr_data(0x0707)    ;

    
      gp_wr_cmm(0x07)       ;	 //Enable the image display.
      gp_wr_data(0x0173)    ;	 
	
      gp_bl_on()            ;	 //Back Light ON
	  
	  delay_ms(20)          ;	 //Delay for 20ms, which is equivalent to two frames.

 }



/* #########********************************************************#############
   #########**** 3.Function Control SPI Touch-Screen (ADS7846)  ****#############
   #########********************************************************#############	*/

  

//*********** CS [P1.31] *************
void tcs_cs_hi(void)
{
  IOSET1 = 0x80000000  ;   //P1.31 =1
}

void tcs_cs_lo(void)
 {
   IOCLR1 = 0x80000000 ;   //P1.31 =0
 }

//*********** SCK [P1.28] **************

void tcs_sck_hi(void)
 {
   IOSET1 = 0x10000000 ;   //P1.28 =1
 }

void tcs_sck_lo(void) 
{
   IOCLR1 = 0x10000000 ;  //P1.28 =0
}


 //*********** Data MOSI [P1.27]************

void tcs_sdo_hi(void)
 {
   IOSET1 = 0x08000000 ;   //P1.27 =1
 }

void tcs_sdo_lo(void)
 {
  IOCLR1 = 0x08000000 ;	   //P1.27 =0
 }



///***********************************************************************
//*******          Function Check Status Press Touch Screen       ********
//************************************************************************
//******** Return Value :        									   ***
//********               Function return value '0'= Not Touched screen ***
//********               Function return value '1'= Touched screen     ***
//************************************************************************


char status_touch(void)
 {
  
  if((IOPIN1 & 0x40000000)== 0x40000000) //Check PEN PIN(P1.30)	
	return 0 ;			  // No press
  else
    return 1 ;			  // Press Touch screen
 }



  //****************************************************************************************
 //******      Function SPI Write and Read Data 1 Byte from Touch Screen ADS7846     ******
 //****************************************************************************************
 //******       Parameter : 												         ******
 //******                  DataByte = data or command control ADS7846 (1Byte)        ****** 
 //******    Return Value : 														 ******
 //******                  Return value adc from touched times 1 byte Pass Function  ******         
 //****************************************************************************************

unsigned char tcs_wr(unsigned char DataByte)			  	
{
  unsigned char Bit,result ;				 	   

  for (Bit = 0; Bit < 8; Bit++)              // 8 Bit Write
      {		
	   	tcs_sck_hi()                     ;	 // Start Clock  High

		if((DataByte & 0x80)== 0x80)
		   tcs_sdo_hi()                  ;   // Set bit data(P0.27) = 1
		else
		   tcs_sdo_lo()                  ;	 // Clear bit data = 0
		   	
		tcs_sck_lo()                     ;   // Clock low
	
		DataByte <<= 1                   ;	 // Shift Next Bit Data
      
	  	//---------- Read Data -------
	    result <<= 1                     ;	
	 											  
	    if ((IOPIN1 & 0x04000000)== 0x04000000)  // If SDI = "1"  (MISO:P0.26)						
	     {
	       result |= 0x01                ;       // Keep Result of Read-data  									
	     }	  
	  }

	  return (result)                    ;
}



//******************************************************************************** 
//******    Function Read X-Y-Position ADC Touch Screen-12 bit adc          ******
//********************************************************************************
//******        Parameter :	No                                              ******
//******    Return Value  :													******
//******                    dif_adc_X = Keep Result ADC X-Position(12 bit)  ******
//******                    dif_adc_Y = Keep result ADC Y-Position(12 bit)  ******                      
//********************************************************************************

void tcs_read(void)
 {
    long tcs_adc_X ,tcs_adc_Y       ;
	unsigned char buf_data[4]       ;

	if(status_touch())
	{
	//--- Set default Pin control Touch Screen ----
	  tcs_sdo_lo() ;
	  tcs_sck_lo() ;
	  tcs_cs_lo () ;


      buf_data[0] = tcs_wr(0xD0)            ;	//Write Command Measure X-Position 
	  buf_data[0] = tcs_wr(0x00)            ;	//Read ADC data X-Position (7-bit byte High) data: 0ddddddd	(bit) 
	  buf_data[1] = tcs_wr(0x90)            ;	//Write Command Measure Y-Position ; Read ADC data X-Position (5-bit byte Low)  data:ddddd000(bit)
	  buf_data[2] = tcs_wr(0x00)            ;	//Read ADC data Y-Position(7-bit byte High) data: 0ddddddd (bit)
	  buf_data[3] = tcs_wr(0x00)            ;	//Read ADC data Y-Position(5-bit byte Low)  data: ddddd000 (bit)

	  tcs_adc_X  = buf_data[0]              ;	// Mark ADC Data X-Position 12 bit
	  tcs_adc_X  = tcs_adc_X << 5           ;
	  tcs_adc_X |= buf_data[1] >> 3         ;
	  tcs_adc_X  = tcs_adc_X & 0x00000FFF   ;
	 
	  tcs_adc_Y  = buf_data[2]              ;   // Mark ADC Data Y-Position 12 bit
	  tcs_adc_Y  = tcs_adc_Y << 5           ;   // Shift 7 bit High
	  tcs_adc_Y |= buf_data[3] >> 3         ;   // Shift 5 bit low
	  tcs_adc_Y  = tcs_adc_Y & 0x00000FFF   ;   // total ADC data 12 bit
	
	  tcs_cs_hi()                           ;
	 
	  dif_adc_X = 4095-tcs_adc_Y	        ;  //ADC 12 bit :LCD Start Landscape
	  dif_adc_Y = tcs_adc_X                 ;	
   }
 }


//******************************************************************
//**              Function Calibrate Touch Green                  **
//**  Touch at Point + on Screen	3 Point(3 step)	for Caribrate **
//******************************************************************
//**      Parameter : No                                          **
//**   Return value : No                                          **
//**              												  **
//******************************************************************

void touch_calibrate(void)	
{									
	char n,touch_mark_point=0;

	plot_mark(25,215,32,222,BRIGHT_RED)                                     ; //Plot Signe + for Position Calibrate Touch Point1
	lcd_printStr("<Touch Screen Calibratetion>",45,164,BRIGHT_BLUE,BLACK)   ; //Plot Text	Color Bright blue
	lcd_printStr("        STEP 1 of 3         ",45,144,BRIGHT_BLUE,BLACK)   ;
    lcd_printStr(" Click on center of '+' icon",45,104,WHITE,BLACK)         ; //Plot Text	Color Black
	lcd_printStr(" for Touch screen calibrate ",45,84,WHITE,BLACK)          ;


	while(touch_mark_point != 3)	        //Number Mark Point Calibrate = 3 Point  
	{
		if(status_touch())                 	//return (1) if touch screen is detected.
		{	  
			for(n=0;n<50;n++)
			 {
				tcs_read()                ;	//Read portrait(ADC X,Y) by touch value from ADS7846 and update dif_adc_X & dif_adc_Y
				buf_adc_X[n] = dif_adc_X  ;	//General buffer for averaging and filter touch screen adc read outs.
				buf_adc_Y[n] = dif_adc_Y  ;	//General buffer for averaging and filter touch screen adc read outs.
			 }
			ave_x_y(50)                   ;	//Take average out of 50 touch measurements, excluding max and min measurement.

			tcs_ave_X[touch_mark_point]=  ave_adc_X ;
			tcs_ave_Y[touch_mark_point]=  ave_adc_Y ;
		
			 
			touch_mark_point++            ;	//touch_cal_status;	//0=xy1 cal, 1=xy2 cal, 2=xy3 cal, 3=calibration success. 0xff=calibration not performed.
		
			while(status_touch()){;}        //Waite until not Touch screen ,

			switch(touch_mark_point)		//Select Caribrate Next Step 2 and 3 
			 {
					case 1:
					
						bg_color(BLACK)                                                        ;   //Clear Screen
						plot_mark(280,120,287,127,BRIGHT_RED)                                  ;   //Plot Signe + for Position Calibrate Touch Point2

                       	lcd_printStr("<Touch Screen Calibratetion>",45,164,BRIGHT_BLUE,BLACK)  ;   //Plot Text Color Bright blue
                      	lcd_printStr("        STEP 2 of 3         ",45,144,BRIGHT_BLUE,BLACK)  ;
                        lcd_printStr(" Click on center of '+' icon",45,104,WHITE,BLACK)        ;   //Plot Text Color white
                     	lcd_printStr(" for Touch screen calibrate ",45,84,WHITE,BLACK)         ;
 


						break;
					case 2:
					
						bg_color(BLACK)                                                         ;   //Clear Screen
						plot_mark(153,24,160,31,BRIGHT_RED)                                     ;	//Plot Signe + for Position Calibrate Touch Point3
						
						lcd_printStr("<Touch Screen Calibratetion>",45,164,BRIGHT_BLUE,BLACK)   ;	//Plot Text	Color Bright blue
                      	lcd_printStr("        STEP 3 of 3         ",45,144,BRIGHT_BLUE,BLACK)   ;
                        lcd_printStr(" Click on center of '+' icon",45,104,WHITE,BLACK)         ;	//Plot Text	Color white
                     	lcd_printStr(" for Touch screen calibrate ",45,84,WHITE,BLACK)          ;
                          

						break;
					case 3:
					
						bg_color(BLACK)                                                         ;   //Clear Screen

						lcd_printStr("<Touch Screen Calibratetion>",45,159,BRIGHT_BLUE,BLACK)   ;	//Plot Text	Color Bright blue
                      	lcd_printStr("          Succeded!         ",45,119,BRIGHT_GREEN,BLACK)  ;
                        lcd_printStr("   -Tap Screen for exit-    ",45,79,WHITE,BLACK)          ;

                       
					    while(!status_touch()){;}  
						delay_ms(100);
						bg_color(BLACK)                                                         ; // Clear Screen

                    
						break;
			}
			 
		}
	 delay_ms(100) ;	
	}
																			 
	set_matrix()   ;  // Calculate Coefficient for use find Position Address real on Display
	

	//You can Keep Value data in valiabie devider,An,Bn,Cn,Dn,En,Fn to E2Prom  so that not 
	//come back	 Caribrate.


//  printf("\n\r divider = %d",divider) ;
//	printf("\n\r      An = %d",An) ;
//	printf("\n\r      Bn = %d",Bn) ;
//	printf("\n\r      Cn = %d",Cn) ;
//  printf("\n\r      Dn = %d",Dn) ;
//	printf("\n\r      En = %d",En) ;
//	printf("\n\r      Fn = %d",Fn) ;
  
 
  }




//*********************************************************************************************
//******  Function Calculate average of measurements,by cut value max and min measurement. ****
//*********************************************************************************************
//******      Parameter :					                                          *********
//******			     num = number times of measurement adc from touch screen      *********
//******    Return value:														      *********
//******                 ave_adc_X = Keep result average adc X-Position			      *********
//******                 ave_adc_Y = Keep result average adc Y-Position               *********     
//*****************************************************************************************


void ave_x_y(unsigned char num)				
{
	unsigned char i;
	unsigned long sum_x, sum_y;
	unsigned long min_x, min_y, max_x, max_y;

	sum_x=0;sum_y=0;min_x=0xffff;min_y=0xffff,max_x=0;max_y=0;

	for(i=0;i<num;i++)
	{
		sum_x += buf_adc_X[i];
		sum_y += buf_adc_Y[i];

		if(buf_adc_X[i]< min_x)		                         //search for minimum X value
		{
			min_x = buf_adc_X[i];
		}
		else if (buf_adc_X[i]> max_x)	                    //search for maximum X value
		{
			max_x = buf_adc_X[i];
		}

		if(buf_adc_Y[i]< min_y)	  	                        //search for minimum Y value
		{
			min_y = buf_adc_Y[i];
		}
		else if(buf_adc_Y[i]> max_y)	  	                //search for maximum Y value
		{
			max_y = buf_adc_Y[i];
		}
	}
	ave_adc_X = (sum_x - min_x - max_x ) / (num-2) ;	//update x average value
	ave_adc_Y = (sum_y - min_y - max_y ) / (num-2) ;	//update y average value
}


//****************************************************************************
//********* Function Calculate Matrix Find value Coeffcient            *******
//****************************************************************************
//*********     Parameter  : No                                        *******
//*********   Return Value : Keep value Coeffcient for use calculate   *******                                      ***********
//*********                  X-Y-Position in variable divider,An,Bn,Cn *******                   
//*********                  Dn,En,Fn                                  *******
//****************************************************************************

 void set_matrix()
  {
	divider =  	((tcs_ave_X[0]-tcs_ave_X[2])*(tcs_ave_Y[1]-tcs_ave_Y[2]))-
	            ((tcs_ave_X[1]-tcs_ave_X[2])*(tcs_ave_Y[0]-tcs_ave_Y[2]))      ;


         An = 	((dis_XD[0]-dis_XD[2])*(tcs_ave_Y[1]-tcs_ave_Y[2]))-
				((dis_XD[1]-dis_XD[2])*(tcs_ave_Y[0]-tcs_ave_Y[2]))            ;

	 	 Bn = 	((tcs_ave_X[0]-tcs_ave_X[2])*(dis_XD[1]-dis_XD[2]))-
				((dis_XD[0]-dis_XD[2])*(tcs_ave_X[1]-tcs_ave_X[2]))            ;

		 Cn =  	(tcs_ave_X[2]*dis_XD[1]-tcs_ave_X[1]*dis_XD[2])*tcs_ave_Y[0]+
		        (tcs_ave_X[0]*dis_XD[2]-tcs_ave_X[2]*dis_XD[0])*tcs_ave_Y[1]+
			    (tcs_ave_X[1]*dis_XD[0]-tcs_ave_X[0]*dis_XD[1])*tcs_ave_Y[2]   ;

		 Dn =   ((dis_YD[0]-dis_YD[2])*(tcs_ave_Y[1]-tcs_ave_Y[2]))-
				((dis_YD[1]-dis_YD[2])*(tcs_ave_Y[0]-tcs_ave_Y[2]))            ;

		 En =   ((tcs_ave_X[0]-tcs_ave_X[2])*(dis_YD[1]-dis_YD[2]))-
				((dis_YD[0]-dis_YD[2])*(tcs_ave_X[1]-tcs_ave_X[2]))            ;

		 Fn =  	(tcs_ave_X[2]*dis_YD[1]-tcs_ave_X[1]*dis_YD[2])*tcs_ave_Y[0]+
		        (tcs_ave_X[0]*dis_YD[2]-tcs_ave_X[2]*dis_YD[0])*tcs_ave_Y[1]+
			    (tcs_ave_X[1]*dis_YD[0]-tcs_ave_X[0]*dis_YD[1])*tcs_ave_Y[2]   ;

  }


//****************************************************************************
//*********** Function Get Position Address real of Display ******************
//****************************************************************************
//***         Parameter  :   										      ****
//***                     num = Number times for Measure adc from Touched **** 
//***       Return Value :											      ****
//***                       X = Keep value address X-Position		      ****
//***                       Y = Keep value address Y-Position             **** 
//****************************************************************************

void get_point(char num)
 {
     char nm ;
	  
	
		  nm=0                               ;
		  while(status_touch() && nm<num)	   //Measure touch x,y num times if the Touch kept pressed,
		   {
			  tcs_read()                     ; //Read value ADC Touch X-Y 
			  buf_adc_X[nm]=	dif_adc_X    ; //keep value ADC Touch-X
		      buf_adc_Y[nm]=	dif_adc_Y    ; //keep value ADC Touch-Y

 			  nm++                           ; 
		   }

		  if(nm==num)	                       //if the touch xy successfuly collected,
		   {	
				ave_x_y(num)                 ; //take average out of 10 touch measurements, excluding max and min measurement.
			    
				dif_adc_X = ave_adc_X        ; //keep average ADC Touch-X
				dif_adc_Y = ave_adc_Y        ; //keep average ADC Touch-Y
               
                X = ((An*dif_adc_X)+(Bn*dif_adc_Y)+Cn)/divider  ; 
	            Y = ((Dn*dif_adc_X)+(En*dif_adc_Y)+Fn)/divider  ;

		   }
	
 }



/*##########*********************************************###########
  ##########*****                                  ******###########
  ##########*****  4.Function Application of GLCD  ******###########
  ##########*****                                  ******###########
  ##########*********************************************###########*/


//***********************************************************************
//**         Function Set  Background color or Clear Screen            **
//***********************************************************************
//**                 Parameter :							           **
//**                            bg_color =  BackGround color of Screen **
//***********************************************************************


void bg_color(long bg_color)
 {	
    long cnt ;
 
    gp_wr_cmm(0x20)           ;    //Command Horizontal GRAM Address Set
    gp_wr_data(0x0000)        ;	   //Address Start 0x0000 
    gp_wr_cmm(0x21)           ;    //Command Vertical GRAM Address Set
    gp_wr_data(0x0000)        ;	   //Address Start 0x0000  
    
	gp_wr_cmm(0x22);
    for(cnt=0;cnt<(320 * 240);cnt++)
      {
        gp_wr_data(bg_color)  ;    // Back-Ground Color Black
      } 
 }


//**********************************************************
//*****             Function Plot Signe+               *****
//**********************************************************
//***** Paramiter:                                     *****
//*****           x1,y1 = Position start Line '-'(Hor) *****
//*****                   plot from Left to Right      *****
//*****           x2,y2 = Position start Line '|'(Ver) *****
//*****                   plot from botton to top      *****
//**********************************************************

void plot_mark(long x1,long y1,long x2,long y2,long color)
 {
    char i ;

  	//---- Line- ----
     gp_wr_cmm(0x20)         ;  //Command Set Adddress Hor(X)
     gp_wr_data(y1)          ;  //Sent X_address  CGRAM
	 		
     gp_wr_cmm(0x21)         ;  //Command Set Address Ver(Y)
     gp_wr_data(x1)          ;  //Sent Y_address  CGRAM
	 
	 gp_wr_cmm(0x22)         ;  //Command Write Data RGB   
	 for(i=0;i<15;i++) 
       gp_wr_data(color)     ;

	//------Line | --------
      gp_wr_cmm(0x03)        ;   //Command Configure Entry Mode
      gp_wr_data(0x1220)     ; 	 //Color:RGB ,Incremen Ver & decrement Hor. address,Address update Ver.

	 gp_wr_cmm(0x20)         ;   //Command Set Adddress Hor(X)
     gp_wr_data(y2)          ;   //Sent X_Address CGRAM 
	 		
     gp_wr_cmm(0x21)         ;   //Command Set Address Ver(Y)
     gp_wr_data(x2)          ;	 //Sent Y_Address CGRAM 
	 
	 gp_wr_cmm(0x22)         ;   //Command Write Data RGB   
	 for(i=0;i<15;i++) 
       gp_wr_data(color)     ;
 
	 gp_wr_cmm(0x03)         ;   //Command Configure Entry Mode
     gp_wr_data(0x1228)      ; 	 //Color:RGB ,Incremen Hor.&Ver.address,Address update Hor.
 	
 }

//*************************************************************
//**                Function Plot Dot 1 Pixel                **
//*************************************************************
//**      Parameter :										 **
//**                 Xadd,Yadd = Position X,Y for Plot Dot   **
//**                     color = Color of dot                **
//*************************************************************

 void plot_dot(unsigned long Xadd,unsigned long Yadd,unsigned long color)
  {
	
	 gp_wr_cmm(0x20)         ;  //Command Set Adddress Hor(X)
     gp_wr_data(Yadd)        ;  //Sent X_address for CGRAM 
		
     gp_wr_cmm(0x21)         ;  //Command Set Address Ver(Y)
     gp_wr_data(Xadd)        ;  //Sent Y_address for CGRAM 
	  			
     gp_wr_cmm(0x22)         ;  //Command Write Data RGB

     gp_wr_data(color)       ;  //write data
 }


//************************************************************
//*******               Function Plot TAB             ********
//************************************************************
//******* Parameter :                                 ********
//*******			  color = color for tab           ********
//************************************************************	     

 void plot_tab(unsigned long color)
  {
    char rx,ry  ;
	  
	for(ry=0;ry<26;ry++)
	 { 
	  gp_wr_cmm(0x20)         ;  //Command Set Adddress Hor(X)
      gp_wr_data(226-ry)      ;  //Sent X_address for CGRAM 
	  gp_wr_cmm(0x21)         ;  //Command Set Address Ver(Y)
      gp_wr_data(309)         ;  //Sent Y_address for CGRAM 			
      gp_wr_cmm(0x22)         ;  //Command Write Data RGB
	
	  for(rx=0;rx<6;rx++) 
        gp_wr_data(color)     ;  //write data
	}

  }


//********************************************************************************
//**                Function Print Text 1 Charecter size 7x11                   **
//********************************************************************************
//**    Parameter :  															**
//**               row      = Ascii Code (Position buffer keep text)		    **
//**               adx,ady  = Position X,Y for begin plot text by will 			**
//**                          begin plot from bottom left to top left   		**
//**               fg_clr   = Color of text										**
//**               bg_clr   = Color background of text(if bg_clr = no_bg or 1=	**
//**                          non color background)								**
//********************************************************************************


 void text_7x11(char row,long adx,long ady,long fg_clr,long bg_clr)
  {
   long ax,ay         ;
   char m,n,tx        ;
     
 
	 ax = adx         ;
	 ay = ady         ; 

	 row = row-0x20   ;

  //------- Print Text 1 Charecter(data 14 Byte) -------------
	  
	  for(m=0;m<14;m++)
	   {
	    //---Sent data byte1=8 bit----

        tx = ascii_7x11[row][m]     ;  //Read data Ascii
	
		for(n=0;n<8;n++)		       //Loop Sent data  1 byte(8bit)
		 {
	      if(tx & 0x80)				   //if data bit7 = 1 ,Plot Color area Charecter
			{              
			  gp_wr_cmm(0x20)       ;  //Command Set Adddress Hor(X)
              gp_wr_data(ay)        ;  //Sent X_Address CGRAM	 		
              gp_wr_cmm(0x21)       ;  //Command Set Address Ver(Y)
              gp_wr_data(ax)        ;  //Sent Y_Address CGRAM
			  gp_wr_cmm(0x22)       ;  //Command Write data 
              gp_wr_data(fg_clr)    ;
 
			}
		  else						   //if data bit7 = 0 ,Plot Color area back ground Charecter
			{
			  if(bg_clr != 1)     
              {
			   gp_wr_cmm(0x20)       ;  //Command Set Adddress Hor(X)
               gp_wr_data(ay)        ;  //Sent X_Address CGRAM	
               gp_wr_cmm(0x21)       ;  //Command Set Adddress Ver(Y)
               gp_wr_data(ax)        ;  //Sent Y_Address CGRAM
			   gp_wr_cmm(0x22)       ;  //Command Write data
               gp_wr_data(bg_clr)    ;  //Sent Data
			  }
			}

		   tx <<= 1                 ;  //Shift Right data 1 bit
		   ay   = ay+1              ;  //Increment Y-address
		 } 

		 m++                        ;  //Increment Next pointter byte Data 


		  //------- Sent data byte2=3 bit -----

		 tx = ascii_7x11[row][m]    ;  //Read data byte2
	
		for(n=0;n<3;n++)			   //Loop sent data byte2 = 3 bit
		 {						    ;
	      if(tx & 0x80)				   //if data bit7 = 1 ,Plot Color area Charecter
			{              
			  gp_wr_cmm(0x20)       ;  //Command Set Adddress Hor(X)
              gp_wr_data(ay)        ;  //Sent X_Address CGRAM		
              gp_wr_cmm(0x21)       ;  //Command Set Adddress Ver(Y)
              gp_wr_data(ax)        ;  //Sent Y_Address CGRAM
			  gp_wr_cmm(0x22)       ;  //Command Write data
              gp_wr_data(fg_clr)    ;
 
			}
		  else						   //if data bit7 = 0 ,Plot Color area back ground Charecter
			{
			  if(bg_clr != 1)     
              {
			   gp_wr_cmm(0x20)       ;  //Command Set Adddress Hor(X)
               gp_wr_data(ay)        ;  //Sent X_Address CGRAM	 		
               gp_wr_cmm(0x21)       ;  //Command Set Adddress Ver(Y)
               gp_wr_data(ax)        ;  //Sent Y_Address CGRAM
			   gp_wr_cmm(0x22)       ;  //Command Write data
               gp_wr_data(bg_clr)    ;
			  }
			}

		   tx <<= 1                 ;  //Shift Right data 1 bit
		   ay = ay+1                ;  //Increment Y-address
		 } 

		ax = ax+1                   ; //Complet sent data 2 byte(11bit) Increment X-Address
		ay = ady                    ; //Set Position Y-address old
 
	  }	


	 //----Fill Back ground Color Position space between Charecter 1 Colum -------
 
     if(bg_clr != 1)     
     {
	  for(n=0;n<11;n++)
	  {
	    gp_wr_cmm(0x20)            ;  //Command Set Adddress Hor(X)
        gp_wr_data(ay)             ;  //Sent X_Address CGRAM	 		
        gp_wr_cmm(0x21)            ;  //Command Set Adddress Ver(Y)
        gp_wr_data(ax)             ;  //Sent Y_Address CGRAM
	    gp_wr_cmm(0x22)            ;  //Command Write data
        gp_wr_data(bg_clr)         ;
		ay = ay+1                  ;  //Increment Y-Address
	  }
     }
  }


 //************************************************************************
 //**                         Function Print String                      **
 //************************************************************************
 //**      Parameter :													 **
 //**                 *str = Charecter ASCII (String)					 **
 //**          cur_x,cur_y = Position X,Y for begin plot text by will 	 **
 //**                        begin plot from bottom left to top left     **
 //**             fg_color = color of Text								 **
 //**             bg_color = color Background of text                    **											
 //************************************************************************

void lcd_printStr(char *str,long cur_x,long cur_y,long fg_color,long bg_color)
{
  unsigned char i;

	for (i=0; str[i] != '\0'; i++)
	{					 
	 text_7x11(str[i],cur_x,cur_y,fg_color,bg_color);
	 cur_x += 8 ; 
		
	}
 }
 









